/* corr.c (Pearson correlation coeffient) */

/***********************************************************************
*  This code is part of Statistics for MySQL.
*
*  Copyright (C) 2011 Heinrich Schuchardt (xypron.glpk@gmx.de)
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*      http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
***********************************************************************/

/**
 * @file corr.c
 * @brief Pearson correlation coeffient.
 *
 * CREATE AGGREGATE FUNCTION corr RETURNS REAL SONAME 'libsqlstat.so';<br />
 * SELECT corr(x,y) FROM mytab;<br />
 * DROP FUNCTION corr;
 */

#include "sqlstat.h"

/**
 * @brief Storage for corr function.
 **/
struct corr_storage {
  int     argc;  /**< @brief number of arguments */
  double  count; /**< @brief Counter. */
  double  sumX;  /**< @brief Sum of x */
  double  sumXX; /**< @brief Sum of x^2 */
  double  sumXY; /**< @brief Sum of x * y */
  double  sumY;  /**< @brief Sum of y */
  double  sumYY; /**< @brief Sum of y^2 */
};

/**
 * \fn my_bool corr_init(UDF_INIT *initid, UDF_ARGS *args, char *message)
 * @brief Called before first usage of function.
 *
 * Check parameters.<br />
 * Indicate type of return value.<br />
 * Allocate memory for counter and sums.
 * @param initid function description
 * @param args parameters
 * @param message error message
 * @return success = 0
 */
my_bool corr_init(UDF_INIT *initid, UDF_ARGS *args, char *message) {
  struct corr_storage * data;
  
  if (args->arg_count < 2 || args->arg_count > 3) {
    strcpy(message,"corr() requires two or three arguments");
    return 1;
  }
  args->arg_type[0] = REAL_RESULT;
  args->arg_type[1] = REAL_RESULT;
  if (args->arg_count > 2) {
    args->arg_type[2] = REAL_RESULT;
  }

  data = (struct corr_storage *) malloc( sizeof(struct corr_storage));
  if (data == NULL) {
    strcpy(message,"Couldn't allocate memory");
    return 1;
  }
  data->argc = args->arg_count;
  
  initid->maybe_null = 1;
  initid->decimals   = NOT_FIXED_DEC;
  initid->max_length = 13 + initid->decimals;
  initid->ptr        = (char *) data;
  initid->const_item = 0;
  
  return 0;
}

/**
 * \fn void corr_reset(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error)
 * @brief Reset function and add first group member
 * Calls clear and add.
 * @param initid function description
 * @param args argument values
 * @param is_null
 * @param error error message
 * @deprecated Reset is only called by MySQL 4.0 and below. It is implemented for backwards
 * compatibility.
 */
void corr_reset(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error) {
  corr_clear(initid, is_null, error);
  corr_add(initid, args, is_null, error);
}

/**
 * \fn void corr_clear(UDF_INIT *initid, char *is_null, char *error)
 * @brief Called at start of group.
 *
 * The counter and sums are cleared. 
 * @param initid function description
 * @param is_null
 * @param error error message
 */
void corr_clear(UDF_INIT *initid, char *is_null, char *error) {
  struct corr_storage *data;
  data = (struct corr_storage *) initid->ptr;
  data->count = 0;
  data->sumX  = 0;
  data->sumXX = 0;
  data->sumXY = 0;
  data->sumY  = 0;
  data->sumYY = 0;
}

/**
 * \fn void corr_add(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error)
 * @brief Add a member of the group.
 *
 * The counter and sums are updated
 * @param initid function description
 * @param args argument values
 * @param is_null
 * @param error error message
 */
void corr_add(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error) {
  struct corr_storage * data;
  double w;
  double x;
  double y;
  
  if (!args->args[0] || !args->args[1]) {
    return;
  }
  data = (struct corr_storage *) initid->ptr;
  if (data->argc > 2) {
    if (!args->args[2]) {
      return;
    }
    w = *((double*) args->args[2]);
  } else {
    w = 1.;
  }
  x = *((double*) args->args[0]);
  y = *((double*) args->args[1]);
  
  data->count  += w;
  data->sumX   += w * x;
  data->sumXX  += w * x * x;
  data->sumXY  += w * x * y;
  data->sumY   += w * y;
  data->sumYY  += w * y * y;
}

/**
 * \fn double corr(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error)
 * @brief Retrieve correlation coefficient. Called at end of group.
 * @param initid function description
 * @param args argument values
 * @param is_null result is NULL
 * @param error error message
 * @return correlation coefficient
 */
double corr(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error) {
  struct corr_storage * data;
  double ret;
  double nx;
  double ny;

  data = (struct corr_storage *) initid->ptr;

  nx = data->count * data->sumXX - data->sumX * data->sumX;
  ny = data->count * data->sumYY - data->sumY * data->sumY;
  
  if (nx <= 0 || ny <= 0) {
    *is_null = 1;
    return 0;
  }
  
  ret = (data->count * data->sumXY - data->sumX * data->sumY)
      / sqrt(nx * ny);
  
  return ret;
}

/**
 * \fn corr_deinit(UDF_INIT *initid)
 * @brief Called after last access to function.
 *
 * Deallocate memory for counter and sums.
 * @param initid
 */
void corr_deinit(UDF_INIT *initid) {
  if (initid->ptr) {
    free(initid->ptr);
  }
}
