/* rand_mt.cc (rand_mt) */

/***********************************************************************
*  This code is part of Statistics for MySQL.
*
*  Copyright (C) 2011 Heinrich Schuchardt (xypron.glpk@gmx.de)
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*      http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
***********************************************************************/

/**
 * @file rand_mt.cc
 * @brief Generate uniformly distributed random number from interval [0,1[.
 *
 * <PRE>
 * CREATE FUNCTION rand_mt RETURNS REAL SONAME 'libsqlstat.so';
 * SELECT rand_mt() FROM mytab;
 * DROP FUNCTION rand_mt;
 * </PRE>
 **/

#include "sqlstat.h"
#include "sqlrand.h"

using sqlstat::MersenneTwister;

/**
 * \fn my_bool rand_mt_init(UDF_INIT *initid, UDF_ARGS *args, char *message)
 * @brief Called before first usage of function.
 *
 * Checksparameters.<br />
 * Indicate type of return value.
 * @param initid function description
 * @param args parameters
 * @param message error message
 * @return success = 0
 */
my_bool rand_mt_init(UDF_INIT *initid, UDF_ARGS *args, char *message) {

  if (!sqlstat_plugin_isloaded()) {
    strcpy(message,"libsqlstat plugin is not loaded. "
      "Use 'INSTALL PLUGIN' for installation.");
    return 1;
  }
  
  if (args->arg_count != 0) {
    strcpy(message,"rand_mt() requires no argument");
    return 1;
  }

  initid->maybe_null = 0;
  initid->decimals   = NOT_FIXED_DEC;
  initid->max_length = 13 + initid->decimals;
  initid->ptr        = NULL;
  initid->const_item = 0;
  return 0;
}

/**
 * \fn double rand_mt(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error)
 * @brief Retrieve Mersenne Twister random number.
 *
 * A double random number in the range [0,1[ is generated. It has 53 random
 * bits.
 * @param initid function description
 * @param args argument values
 * @param is_null result is NULL
 * @param error error message
 * @return random number
 */
double rand_mt(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error) {
  return MersenneTwister::nextDouble();
}

/**
 * \fn rand_mt_deinit(UDF_INIT *initid)
 * @brief Called after last access to function.
 *
 * Deallocate memory for counter and sums.
 * @param initid
 */
void rand_mt_deinit(UDF_INIT *initid) {
  return;
}
