/* rand_norm.cc (rand_norm) */

/***********************************************************************
*  This code is part of Statistics for MySQL.
*
*  Copyright (C) 2011 Heinrich Schuchardt (xypron.glpk@gmx.de)
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*      http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
***********************************************************************/

/**
 * @file rand_norm.cc
 * @brief Generate random number from standard normal distribution.
 *
 * CREATE FUNCTION rand_norm RETURNS REAL SONAME 'libsqlstat.so';
 * SELECT rand_norm() FROM mytab;
 * DROP FUNCTION rand_norm;
 **/

#include "sqlstat.h"
#include "sqlrand.h"

using sqlstat::MersenneTwister;

/**
 * \fn my_bool rand_norm_init(UDF_INIT *initid, UDF_ARGS *args, char *message)
 * @brief Called before first usage of function.
 *
 * Checksparameters.<br />
 * Indicate type of return value.
 * @param initid function description
 * @param args parameters
 * @param message error message
 * @return success = 0
 */
my_bool rand_norm_init(UDF_INIT *initid, UDF_ARGS *args, char *message) {
  int i;

  if (!sqlstat_plugin_isloaded()) {
    strcpy(message,"libsqlstat plugin is not loaded. "
      "Use 'INSTALL PLUGIN' for installation.");
    return 1;
  }
  
  if (args->arg_count > 2 || args->arg_count < 0) {
    strcpy(message,"rand_norm() takes up to two arguments");
    return 1;
  }

  for ( i = 0; i < (int) args->arg_count; i++) {
    args->arg_type[i] = REAL_RESULT;
  }

  initid->maybe_null = 0;
  initid->decimals   = NOT_FIXED_DEC;
  initid->max_length = 13 + initid->decimals;
  initid->ptr        = NULL;
  initid->const_item = 0;
  return 0;
}

/**
 * \fn double rand_norm(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error)
 * @brief Retrieve a variate of the normal distribution.
 *
 * The default value for the mean \f$\mu\f$ is 0, the default value of the standard
 * deviation \f$\sigma\f$ is 1.
 * @param initid function description
 * @param args argument values
 * @param is_null result is NULL
 * @param error error message
 * @return random number
 */
double rand_norm(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error) {
  double mu = 0;
  double sigma = 1;  

  if (args->arg_count >= 1) {
    if (!args->args[0]) {
      *error = 1;
    } else {
    sigma = *((double*) args->args[0]);
    }
  }
  if (args->arg_count >= 2) {
    if (!args->args[1]) {
      *error = 1;
    } else {
    mu = *((double*) args->args[1]);
    }
  }
  return mu + sigma * MersenneTwister::gaussian();
}

/**
 * \fn rand_norm_deinit(UDF_INIT *initid)
 * @brief Called after last access to function.
 * 
 * @param initid
 */
void rand_norm_deinit(UDF_INIT *initid) {
}
