/* rownumber.cc (Row number) */

/***********************************************************************
*  This code is part of Statistics for MySQL.
*
*  Copyright (C) 2011 Heinrich Schuchardt (xypron.glpk@gmx.de)
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*      http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
***********************************************************************/

/**
 * @file rownumber.c
 * @brief Determine row number.
 *
 * CREATE FUNCTION rownumber RETURNS INTEGER SONAME 'libsqlstat.so';<br />
 * SELECT rownumber() FROM mytab;<br />
 * DROP FUNCTION rownumber;
 **/

#include "sqlstat.h"

/**
 * @brief Storage for rownumber function.
 **/
struct rownumber_storage {
  long long count; /**< @brief Counter. */
};


/**
 * \fn my_bool rownumber_init(UDF_INIT *initid, UDF_ARGS *args, char *message)
 * @brief Called before first usage of function.
 *
 * Checksparameters.<br />
 * Indicate type of return value.
 * @param initid function description
 * @param args parameters
 * @param message error message
 * @return success = 0
 */
my_bool rownumber_init(UDF_INIT *initid, UDF_ARGS *args, char *message) {
  struct rownumber_storage *data;

  if (args->arg_count != 0) {
    strcpy(message,"rownumber() requires no argument");
    return 1;
  }

  data = (struct rownumber_storage *) malloc( sizeof(struct rownumber_storage));
  if (data == NULL) {
    strcpy(message,"Couldn't allocate memory");
    return 1;
  }  
  data->count = 0;

  initid->maybe_null = 0;
  initid->decimals   = NOT_FIXED_DEC;
  initid->max_length = 13 + initid->decimals;
  initid->ptr        = (char *) data;
  initid->const_item = 0;
  return 0;
}

/**
 * \fn double rownumber(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error)
 * @brief Retrieve row number.
 *
 * @param initid function description
 * @param args argument values
 * @param is_null result is NULL
 * @param error error message
 * @return row number
 */
long long rownumber(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error) {
  struct rownumber_storage *data;
  long long ret;
  if (initid->ptr) {
    data = (struct rownumber_storage *) initid->ptr;
    ret = ++data->count;
  } else {
    ret = 0;
    *error = 1;
  }
  return ret;
}

/**
 * \fn rownumber_deinit(UDF_INIT *initid)
 * @brief Called after last access to function.
 *
 * Deallocate memory for counter.
 * @param initid
 */
void rownumber_deinit(UDF_INIT *initid) {
  if (initid->ptr) {
    free(initid->ptr);
  }
  return;
}
