/* skewness_pop.c (population moment coefficient of skewness) */

/***********************************************************************
*  This code is part of Statistics for MySQL.
*
*  Copyright (C) 2011 Heinrich Schuchardt (xypron.glpk@gmx.de)
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*      http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
***********************************************************************/

/**
 * @file skewness_pop.c
 * @brief Population moment coefficient of skewness.
 *
 * CREATE AGGREGATE FUNCTION skewness_pop RETURNS REAL SONAME 'libsqlstat.so';
 * SELECT skewness_pop(x) FROM mytab;
 * DROP FUNCTION skewness_pop;
 * */

#include "sqlstat.h"

/**
 * @brief Storage for population moment coefficient of skewness
 */
struct skewness_pop_storage {
  int     argc;  /**< @brief number of arguments */
  double  count;  /**< @brief Counter. */
  double  sumX;   /**< @brief Sum of x */
  double  sumXX;  /**< @brief Sum of x^2 */
  double  sumXXX; /**< @brief Sum of x^3 */
};

/**
 * \fn my_bool skewness_pop_init(UDF_INIT *initid, UDF_ARGS *args, char *message)
 * @brief Called before first usage of function.
 *
 * Check parameters.<br />
 * Indicate type of return value.<br />
 * Allocate memory for counter and sums.
 * @param initid function description
 * @param args parameters
 * @param message error message
 * @return success = 0
 */
my_bool skewness_pop_init(UDF_INIT *initid, UDF_ARGS *args, char *message) {
  struct skewness_pop_storage * data;
  
  if (args->arg_count < 1 || args->arg_count > 2) {
    strcpy(message,"skewness_pop() requires one or two arguments");
    return 1;
  }
  args->arg_type[0] = REAL_RESULT;
  if (args->arg_count > 1) {
    args->arg_type[1] = REAL_RESULT;
  }

  data = (struct skewness_pop_storage *) malloc( sizeof(struct skewness_pop_storage));
  if (data == NULL) {
    strcpy(message,"Couldn't allocate memory");
    return 1;
  }  
  data->argc = args->arg_count;
  
  initid->maybe_null = 1;
  initid->decimals   = NOT_FIXED_DEC;
  initid->max_length = 13 + initid->decimals;
  initid->ptr        = (char *) data;
  initid->const_item = 0;
  
  return 0;
}

/**
 * \fn void skewness_pop_reset(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error)
 * @brief Reset function and add first group member
 * Calls clear and add.
 * @param initid function description
 * @param args argument values
 * @param is_null
 * @param error error message
 * @deprecated Reset is only called by MySQL 4.0 and below. It is implemented for backwards
 * compatibility.
 */
void skewness_pop_reset(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error) {
  skewness_pop_clear(initid, is_null, error);
  skewness_pop_add(initid, args, is_null, error);
}

/**
 * void skewness_pop_clear(UDF_INIT *initid, char *is_null, char *error)
 * @brief Called at start of group.
 *
 * The counter and sums are cleared. 
 * @param initid function description
 * @param is_null
 * @param error error message
 */
void skewness_pop_clear(UDF_INIT *initid, char *is_null, char *error) {
  struct skewness_pop_storage *data;
  data = (struct skewness_pop_storage *) initid->ptr;
  data->count = 0;
  data->sumX  = 0;
  data->sumXX = 0;
  data->sumXXX = 0;
}

/**
 * \fn void skewness_pop_add(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error)
 * @brief Add a member of the group.
 *
 * The counter and sums are updated
 * @param initid function description
 * @param args argument values
 * @param is_null
 * @param error error message
 */
void skewness_pop_add(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error) {
  struct skewness_pop_storage * data;
  double x;
  double w;
  
  if (!args->args[0]) {
    return;
  }
  data = (struct skewness_pop_storage *) initid->ptr;
  if (data->argc > 1) {
    if (!args->args[1]) {
      return;
    }
    w = *((double*) args->args[1]);
  } else {
    w = 1.;
  }
  x = *((double*) args->args[0]);
  
  data->count  += w;
  data->sumX   += w * x;
  data->sumXX  += w * x*x;
  data->sumXXX += w * x*x*x;
}

/**
 * \fn double skewness_pop(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error)
 * @brief Retrieve population moment coefficient of skewness. Called at end of group.
 * @param initid function description
 * @param args argument values
 * @param is_null result is NULL
 * @param error error message
 * @return population moment coefficient of skewness
 */
double skewness_pop(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error) {
  struct skewness_pop_storage * data;
  double m3;
  double m2;
  double mean;
  
  double ret;

  data = (struct skewness_pop_storage *) initid->ptr;

  if (data->count <= 0) {
    *is_null = 1;
    return 0;
  }
  
  mean = data->sumX / data->count;
  
  m3 = (data->sumXXX - 3 * mean * data->sumXX + 2 * mean * mean * data->sumX) / data->count;
  m2 = (data->sumXX - mean * data->sumX) / data->count;
  
  if (m2 <= 0) {
    *is_null = 1;
    return 0;
  }
  
  ret = m3 / sqrt(m2 * m2 * m2);
  
  return ret;
}

/**
 * \fn void skewness_pop_deinit(UDF_INIT *initid)
 * @brief Called after last access to function.
 *
 * Deallocate memory for counter and sums.
 * @param initid
 */
void skewness_pop_deinit(UDF_INIT *initid) {
  if (initid->ptr) {
    free(initid->ptr);
  }
}
