/* sqlrand.h (Random numbers) */

/***********************************************************************
*  This code is part of Statistics for MySQL.
*
*  Copyright (C) 2011 Heinrich Schuchardt (xypron.glpk@gmx.de)
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*      http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
***********************************************************************/

/**
 * @file sqlrand.h
 * @brief Definitions for random number generations.
 */
 
#ifndef SQLRAND_H

#define SQLRAND_H 1

#define MT_LEN          624 /**< @brief length of buffer */
#define MT_IA           397 /**< @brief offset in buffer */
#define MT_IB           (MT_LEN - MT_IA) /**< @brief length - offset in buffer */
#define UPPER_MASK      0x80000000       /**< @brief mask for bit 31 */
#define LOWER_MASK      0x7FFFFFFF       /**< @brief mask for low bits */
#define MATRIX_A        0x9908B0DF       /**< @brief mask for distributing bit 0 */
/** @brief merge bit 31 of one parameter with the low bits of the other
 * parameter.
 */
#define TWIST(b,i,j)    ((b)[i] & UPPER_MASK) | ((b)[j] & LOWER_MASK)
/** @brief distribute the low bit */
#define MAGIC(s)        (((s)&1)*MATRIX_A)

namespace sqlstat{
  
/** 
 * @brief Mersenne Twister random number generator
 *
 * This code is based on code written by Michael L. Brundage which he has
 * placed in the public domain (see
 * <a href=
 * "http://www.qbrundage.com/michaelb/pubs/essays/random_number_generation.html"
 * >http://www.qbrundage.com/michaelb/pubs/essays/random_number_generation.html
 * </a>).
 *
 * Matsumoto, Makoto and Nishimura, Takuji,<br />
 * "Mersenne twister: a 623-dimensionally equidistributed uniform pseudo-random 
 * number generator", <br />
 * ACM Transactions on Modeling and Computer Simulation, 1998
 **/
  class MersenneTwister {

  private:
  public:
    /**
     * void MersenneTwister::deinit()
     * @brief Deinitialize the class
     *
     * This method must be called before leaving the program to release all
     * resources and memory associated to the class.
     */
    static void deinit();
    /**
     * @brief Initialize the class.
     *
     * This method must be called be called before using the class.
     *
     * It creates mutexes and initializes the buffer.
     */
    static void init();
    /**
     * @brief Returns random double number.
     * 
     * A double random number in the range [0,1[ is generated. It has 53 random
     * bits.
     */
    static double nextDouble();
    /**
     * @brief Returns random integer number.
     *
     * A random number is retrieved from the buffer. If the buffer is empty
     * new random numbers are generated.
     * @return Random number
     */
    static unsigned long random();
    /**
     * @brief Returns variate of the normal distribution.
     * 
     * A random number is created according to a normal distribution with
     * mean = 0 and standard deviation = 1 using the Marsaglia polar method.
     * The Maraglia method avoids the calculation of sine and cosine found in
     * the Box-Muller transform.
     */
    static double gaussian();
  };
}
#endif // SQLRAND_H
